# -*- coding: utf-8 -*-

import re
from collections import defaultdict

# Mapping nozbe codes to rtm codes
RECUR_2_MILK = {
    '0' : None,
    '1' : u"Every day",
    '2' : u"Every weekday",
    '3' : u"Every week",
    '4' : u"Every 2 weeks",
    '5' : u"Every month",
    '6' : u"Every 6 months",
    '7' : u"Every year",
    }

# Name used for "preserve notes" task
NOTE_TASK_NAME = u"Save this note"

re_badchars = re.compile(u"[^-\w]+", re.UNICODE)
def context_to_tag(ctx_name):
    name = re_badchars.sub("-",ctx_name)
    return u"@" + name

#print context_to_tag("Ala ma kota")
#print context_to_tag(u"Zażółć gęślą jaźń")
#print context_to_tag(u"Komp/Platon")


def import_nozbe_actions(rtm_updater, actions, notes, verbose, dry_run):

    for action in actions:
        name = action['name']
        project_name = action['project_name']

        completed = (str(action.get('completed', 0)) == "1")
        if completed:
            #print (u"Skipping completed action from project %s: %s" % (project_name, name)).encode('utf-8')
            continue

        if dry_run or (project_name == "Inbox"):
            list_id = None
        else:
            list_id = rtm_updater.find_or_create_list(project_name)

        completed = (str(action.get('completed', 0)) == "1")
                
        tags = []
        if action['context_name']:
            tags.append(context_to_tag(action['context_name']))
        if str(action['next']) == "1":
            tags.append("Next")

        due_date = None  # ...
        if action['datetime']:
            due_date = action['datetime']

        repeat = None
        if not completed: # avoid resurrecting old tasks
            repeat = RECUR_2_MILK[ action['recur']  ]

        estimate = None  # 3 days 2 hours 10 minutes
        at = str(action['time'])
        if at != "0":
            estimate = at + " minutes"

        if verbose:
            intro = (completed and "Saving completed task" or "Creating new task")
            print (u"%(intro)s on list %(project_name)s\n   Task name: %(name)s \n   Repeat: %(repeat)s, due: %(due_date)s, estimate: %(estimate)s, tags: %(tags)s" % locals()).encode('utf-8')

        if not dry_run:
            rtm_updater.create_task(
                task_name = name,
                list_id = list_id,
                tags = tags,
                due_date = due_date,
                estimate = estimate,
                repeat = repeat,
                completed = completed)
           # priority, url, notes - unused
    
    # First group notes by project
    project_notes = defaultdict(lambda: [])
    for note in notes:
        project_notes[note['project_name']].append(
            (note['name'], note['body'])
            )

    # ... and save them
    for project_name, here_notes in project_notes.iteritems():
        if dry_run or (project_name == "Inbox"):
            list_id = None
        else:
            list_id = rtm_updater.find_or_create_list(project_name)
        task_name = NOTE_TASK_NAME
        if verbose:
            print "Creating preserve note task on list %(project_name)s. Task name: %(task_name)s\n" % locals()
            print "Notes bound:\n"
            for (title, body) in here_notes:
                print title, "\n"
                print "   ", body.replace("\n", "\n    "), "\n"
        if not dry_run:
            rtm_updater.create_task(
                task_name = task_name,
                list_id = list_id,
                tags = ["Note"],
                due_date = "today",
                notes = here_notes)



# -*- coding: utf-8 -*-

class RtmUpdater(object):
    """
    Class responsible for creating new lists, tags, and tasks.
    Apart from wrapping RememberTheMilk API quirks it handles
    duplicate detection in case of lists and tags.

    All updates are performed inside single timeline (Rtm undo
    context) unless set_undo_point routine is called.
    """
    
    def __init__(self, api):
        """
        Initializes object. 

        Parameters
        ----------

        - `api` - authorized RtmApi object used to handle communication.
        """
        self.api = api
        self._list_cache = None     # name -> id, created on first use
        self._tag_cache = dict()    # name -> id
        self._timeline = None

    def find_or_create_list(self, list_name):
        """
        Looks for the list of given name. If such list does not exist, creates
        it. Returns list_id.
        """
        self._load_list_cache_if_necessary()
        id = self._list_cache.get(list_name)
        if id:
            return id
        timeline = self._get_timeline()

        r = self.api.rtm.lists.add(timeline = timeline,
                                   name = list_name)
        id = r.list.id
        self._list_cache[list_name] = id
        return id

    def known_lists(self):
        """
        Returns all known lists (except smartlists, only true lists are returned).

        Return value is a list of pairs (list_id, list_name)
        """
        self._load_list_cache_if_necessary()
        return [ (id, name) for name, id in self._list_cache.iteritems() ]

    def create_task(self, task_name,
                    list_id = None,
                    tags = [], 
                    completed = False,
                    priority = None,
                    due_date = None,
                    estimate = None,
                    repeat = None,
                    url = None,
                    notes = [],
                    ):
        """
        Creates new task and sets some attributes. Returns pair (taskseries_id, task_id).

        :Parameters:
        - task_name 
        - list_id - list identifier or None for Inbox
        - tags - list of tags (note: names can't contain ',')
        - completed - is task already completed
        - priority - 1, 2, or 3
        - due_date - due date (will be parsed, usually ISO 8601 format should work)
        - estimate - estimated cost (text like "3 days 2 hours 40 minutes")
        - repeat - task repeat. Text like "Every Tuesday", "Every month on the 4th", "Every week until 1/1/2007",
                "After a week" etc. See http://www.rememberthemilk.com/help/answers/basics/repeatformat.rtm
        - url - url related to the task
        - notes - list of pairs (title, text)

        :Return:
        - pair of (taskseries_id, task_id)
        """
        timeline = self._get_timeline()
        if list_id:
            r = self.api.rtm.tasks.add(timeline = timeline,
                                       list_id = list_id,
                                       name = task_name)
        else:
            r = self.api.rtm.tasks.add(timeline = timeline,
                                       name = task_name)
        task_id = r.list.taskseries.task.id
        taskseries_id = r.list.taskseries.id
        if not list_id:
            # addTags and other methods require it
            list_id = r.list.id

        if tags:
            tag_text = ", ".join(tags)
            self.api.rtm.tasks.addTags(
                timeline = timeline,
                list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
                tags = tag_text)

        if completed:
            self.api.rtm.tasks.complete(
                timeline = timeline,
                list_id = list_id, taskseries_id = taskseries_id, task_id = task_id)

        if priority:
            self.api.rtm.tasks.setPriority(
                timeline = timeline,
                list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
                priority = str(priority))

        if due_date:
            self.api.rtm.tasks.setDueDate(
                timeline = timeline,
                list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
                due = due_date, parse = "1", # has_due_time = 1
                )

        if estimate:
            self.api.rtm.tasks.setEstimate(
                timeline = timeline,
                list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
                estimate = estimate,
                )

        if repeat:
            self.api.rtm.tasks.setRecurrence(
                timeline = timeline,
                list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
                repeat = repeat,
                )
            
        if url:
            self.api.rtm.tasks.setURL(
                timeline = timeline,
                list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
                url = url)

        for (note_title, note_text) in notes:
            self.add_task_note(list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
                               note_title = note_title, note_text = note_text)

        return (taskseries_id, task_id)

    def add_task_note(self, 
                      list_id, taskseries_id, task_id,
                      note_title, note_text):
        """
        Adds new note to the task. 

        Parameters
        ----------

        `list_id`, `taskseries_id`, `task_id` - necessary list and task identifiers
        `note_title`, `note_text` - title and text of new note
        """
        timeline = self._get_timeline()
        self.api.rtm.tasks.notes.add(
            timeline = timeline,
            list_id = list_id, taskseries_id = taskseries_id, task_id = task_id,
            note_title = note_title, note_text = note_text)

    def set_undo_point(self):
        # Just forgetting previous timeline. New one will be created before first update
        self._timeline = None

    def _get_timeline(self):
        if self._timeline is None:
            r = self.api.rtm.timelines.create()
            self._timeline = r.timeline.value
        return self._timeline

    def _load_list_cache_if_necessary(self):
        if self._list_cache is None:
            r = self.api.rtm.lists.getList()
            cache = {}
            for l in r.lists:
                if not int(l.smart):
                    cache[l.name] = l.id
            self._list_cache = cache


"""
Useful examples:

    # get all open tasks, see http://www.rememberthemilk.com/services/api/methods/rtm.tasks.getList.rtm
    result = api.rtm.tasks.getList(filter="status:incomplete")
    for tasklist in result.tasks:
        for taskseries in tasklist:
            print taskseries.task.due, taskseries.name

    # Create new list
    result = api.rtm.lists.add(timeline = timeline,
                               name = u"Zażółć gęślą test")
    list_id = result.list.id
    print "Created list with id", list_id

    # And task
    result = api.rtm.tasks.add(timeline = timeline,
                               list_id = list_id,
                               name = u"Żółć gęśą zajaźniam")
    task_id = result.list.taskseries.task.id
    print task_id
    print "Created task", task_id



"""



# -*- coding: utf-8 -*-
# (c) 2010, Marcin Kasperski

from setuptools import setup, find_packages

version = '0.1.3'
long_description = open("README.txt").read()
classifiers = [
    "Programming Language :: Python",
    "Intended Audience :: Developers",
    "Topic :: Software Development :: Libraries :: Python Modules",
    # TODO: Development Status, Environment, Topic
    ]

setup(name='mekk.rtmimport',
      version=version,
      description="Import foreign (at the moment, Nozbe) data to RememberTheMilk",
      long_description=long_description,
      classifiers=classifiers,
      keywords='rtm,RememberTheMilk',
      author='Marcin Kasperski',
      author_email='Marcin.Kasperski@mekk.waw.pl',
      url='',
      license='LGPL',
      package_dir={'':'src'},
      packages=find_packages('src', exclude=['ez_setup', 'examples', 'tests']),
      namespace_packages=['mekk'],
      test_suite = 'nose.collector',
      zip_safe=False,
      install_requires=[
        'simplejson', 
        'RtmAPI>=0.3.3', #http://pypi.python.org/pypi/RtmAPI/0.3.1
        'keyring>=0.3',
      ],
      tests_require=[
          'nose', 
      ],
      entry_points = {
        'console_scripts': [
            'rtmimport = mekk.rtmimport.run:run',
            ],
        },

)
